package com.migao.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.migao.config.response.BasePageRequest;
import com.migao.config.response.PageBeanOne;
import com.migao.config.response.ResponseBean;
import com.migao.entity.po.Supplier;
import com.migao.entity.vo.req.CompanyEvaluationQuerysReq;
import com.migao.entity.vo.req.SupplierInsertReq;
import com.migao.entity.vo.req.SupplierQueryReq;
import com.migao.entity.vo.req.SupplierUpdateReq;
import com.migao.entity.vo.res.CompanyEvaluationQueryRes;
import com.migao.entity.vo.res.DictionaryListRes;
import com.migao.entity.vo.res.SupplierPageQueryRes;
import com.migao.entity.vo.res.SupplierQueryRes;

import java.util.List;

/**
 * @author dingsong
 */
public interface SupplierService  extends IService<Supplier> {
    /**
     * 新增供应商
     * @param supplierInsertReq 供应商信息
     * @return
     */
    public ResponseBean<?> insert(SupplierInsertReq supplierInsertReq);

    /**
     * 通过id删除
     * @param id 供应商id
     * @return
     */
    public ResponseBean<?> deleteById(String id);

    /**
     * 更新供应商
     * @param supplierUpdateReq 供应商信息
     * @return
     */
    public ResponseBean<?> updateById(SupplierUpdateReq supplierUpdateReq);

    /**
     * 查询供应商详情
     * @param id 供应商id
     * @return SupplierQueryRes
     */
    public ResponseBean<SupplierQueryRes> findById(String id);

    /**
     * 分页查看所有供应商信息
     * @param basePageRequest 页数
     * @return
     */
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQuery(BasePageRequest basePageRequest);

    /**
     * 分页查看当前用户供应商信息
     * @param basePageRequest 页数
     * @return
     */
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQueryById(BasePageRequest basePageRequest);

    /**
     * 分页模糊查看所有供应商信息
     * @param supplierQueryReq 关键字
     * @return
     */
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQueryOne(SupplierQueryReq supplierQueryReq);

    /**
     * 分页模糊查看当前用户供应商信息
     * @param supplierQueryReq 关键字
     * @return
     */
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQueryOneById(SupplierQueryReq supplierQueryReq);


    /**
     * 分页查看公司评价
     * @param cReq id集合
     * @return
     */
    public ResponseBean<PageBeanOne<CompanyEvaluationQueryRes>> pageEvaluation(CompanyEvaluationQuerysReq cReq);

    /**
     * 导出所有供应商
     * @return
     */
    public ResponseBean<?> exportSupplier();

    /**
     * 公司类别下拉列表
     * @return
     */
    public ResponseBean<List<DictionaryListRes>> companyTypeClassify();
}
