package com.migao.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.migao.config.constant.*;
import com.migao.config.listener.ExcelListener;
import com.migao.config.properties.SystemProperties;
import com.migao.config.response.PageBean;
import com.migao.config.response.ResponseBean;
import com.migao.config.response.ResponseBuilder;
import com.migao.entity.po.*;
import com.migao.entity.vo.req.*;
import com.migao.entity.vo.res.*;
import com.migao.mapper.*;
import com.migao.service.ProjectService;
import com.migao.util.EntityUtils;
import com.migao.util.FileUtils;
import com.migao.util.ProjectNoUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import sun.net.ConnectionResetException;


import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Slf4j
@Service
@Transactional(rollbackFor = {Exception.class})
public class ProjectServiceImpl implements ProjectService {

    @Resource
    private ProjectRepository projectRepository;

    @Resource
    private ProjectMapper projectMapper;
    @Resource
    private DictionaryMapper dictionaryMapper;
    @Resource
    private ProjectFileMapper projectFileMapper;
    @Resource
    private FileInfoRepository fileInfoRepository;
    @Resource
    private SystemProperties systemProperties;
    @Resource
    private PurchasingApprovalMapper purchasingApprovalMapper;
    @Resource
    private ProjectDemandMapper projectDemandMapper;
    @Resource
    private ContractMapper contractMapper;
    @Resource
    private ProjectStageMapper projectStageMapper;
    @Resource
    private ProjectTypeMapper projectTypeMapper;
    @Resource
    private ProjectTypeStageMapper projectTypeStageMapper;
    @Resource
    private MilestoneMapper milestoneMapper;
    @Resource
    private ProjectMemberMapper projectMemberMapper;
    @Resource
    private DemandLibraryMapper demandLibraryMapper;
    @Resource
    private FinancesMapper financesMapper;
    @Resource
    private CompanyEvaluationMapper companyEvaluationMapper;

    /**
     * 增
     *
     * @param projectInsertReq projectInsertReq
     * @return ?
     */
    @Override
    public ResponseBean<?> insert(ProjectInsertReq projectInsertReq) {
        Project project = new Project();
        BeanUtils.copyProperties(projectInsertReq, project);
        project.setReserveStatus(ProjectStatus.TO_APPROVAL.getName());
        project.setDeleted(0);
        project.setParentId(0);
        project.setStatus(CheckStatus.UNDERWAY.getName());
        project.setNo(ProjectNoUtil.getCode());
        return ResponseBuilder.ok(projectRepository.save(project));
    }

    /**
     * 分页查
     *
     * @param projectPageQueryReq projectPageQueryReq
     * @return ?
     */
    @Override
    public ResponseBean<PageBean<ProjectQueryRes>> pageQuery(ProjectPageQueryReq projectPageQueryReq) {
        //返回储备状态为拒绝/待审批
        Integer page = projectPageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = projectPageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;
        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        if (projectPageQueryReq.getCreateTime() != null) {
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getName())) {
            queryWrapper.eq(Project::getCreateTime, projectPageQueryReq.getCreateTime());
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getOrderStatus())) {
            queryWrapper.eq(Project::getOrderStatus, projectPageQueryReq.getOrderStatus());
        }
        queryWrapper.eq(Project::getReserveStatus, ProjectStatus.TO_APPROVAL.getName()).or().eq(Project::getReserveStatus, ProjectStatus.REJECT.getName());
        Page<Project> all = projectMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    ProjectQueryRes projectQueryRes = ProjectQueryRes
                            .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                            .build();
                    return projectQueryRes;
                }
        );
    }

    /**
     * 删
     *
     * @param id id
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteById(Integer id) {
        projectRepository.findById(id).ifPresent(
                i -> {
                    i.setDeleted(1);
                    projectRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 批量删
     *
     * @param ids ids
     * @return ?
     */
    @Override
    public ResponseBean<?> deleteBatch(List<Integer> ids) {
        projectRepository.findAllById(ids).forEach(
                i -> {
                    i.setDeleted(1);
                    projectRepository.save(i);
                }
        );
        return ResponseBuilder.ok();
    }

    /**
     * 改
     *
     * @param projectUpdateReq projectUpdateReq
     * @return ?
     */
    @Override
    public ResponseBean<?> update(ProjectUpdateReq projectUpdateReq) {
        projectRepository.save(EntityUtils.copyProperties(projectUpdateReq, Project.class));
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<List<ProjectQueryRes>> query(ProjectQueryReq projectQueryReq) {
        return null;
    }


    @Override
    public ResponseBean<List<DictionaryListRes>> projectClassify() {
        String value = "PROJECT_CLASSIFY";
        List<DictionaryListRes> all = dictionaryMapper.findByValue(value);
        return ResponseBuilder.ok(all);
    }

    @Override
    public ResponseBean<List<DictionaryListRes>> specialtyClassify() {
        String value = "SPECIALTY_CLASSIFY";
        List<DictionaryListRes> all = dictionaryMapper.findByValue(value);
        return ResponseBuilder.ok(all);
    }

    @Override
    public ResponseBean<List<DictionaryListRes>> twoClassify() {
        String value = "TWO_CLASSIFY";
        List<DictionaryListRes> all = dictionaryMapper.findByValue(value);
        return ResponseBuilder.ok(all);
    }

    @Override
    public ResponseBean<ProjectQueryRes> findById(Integer id) {
        Project project = projectMapper.selectById(id);
        ProjectQueryRes projectQueryRes = new ProjectQueryRes();
        BeanUtils.copyProperties(project, projectQueryRes);
        return ResponseBuilder.ok(projectQueryRes);
    }

    @Override
    public ResponseBean<?> approve(Integer id, ExamineStatus status) {
        Project project = projectMapper.selectById(id);
        if (project == null) {
            ResponseBuilder.fail("项目不存在");
        }
        if (status.equals(FinancesStatus.PASS)) {
            project.setReserveStatus(ProjectStatus.APPROVE.getName());
            project.setOrderStatus(ProjectStatus.RESERVES_ISSUED.getName());
            projectMapper.updateById(project);
        } else {
            project.setReserveStatus(ProjectStatus.REJECT.getName());
            projectMapper.updateById(project);
        }

        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<?> importData(Integer fileId) {

        //1.读取文件
        List<ImportProjectReq> list = new ArrayList<>();
        Optional<FileInfo> byId =
                fileInfoRepository.findById(fileId);
        byId.ifPresent(
                fileInfo -> {
                    EasyExcel.read(
                            fileInfo.getDiskPath(),
                            ImportProjectReq.class,
                            new ExcelListener<>(list)
                    ).sheet().doRead();
                }
        );
        log.info("读取结果:{}", list.size());
        //去除重复数据
        list.forEach(
                data -> {
                    Project project = EntityUtils.copyProperties(data, Project.class);
                    projectMapper.insert(project);
                }
        );
        //手动提交
        log.info("保存项目储备数量:{}", list.size());
        return ResponseBuilder.ok(list.size());
    }

    @Override
    public ResponseBean<?> export(ProjectPageQueryReq projectPageQueryReq) {
        List<ProjectQueryRes> data = pageQuery(projectPageQueryReq).getData().getRow();
        String fileName = StringUtils.join(
                FileUtils.getFileNo(), SystemConstant.EXCEL_SUFFIX
        );
        String filePath = StringUtils.join(
                systemProperties.getFileStoreDirPath(), fileName
        );
        String fileUrl = StringUtils.join(
                systemProperties.getDownloadPrefix(), fileName
        );
        ExcelWriter excelWriter = EasyExcel.write(filePath, ProjectQueryRes.class).build();
        WriteSheet writeSheet = EasyExcel.writerSheet("项目储备数据").build();
        excelWriter.write(data, writeSheet);
        //关闭流
        excelWriter.finish();
        return ResponseBuilder.ok(fileUrl);
    }

    @Override
    public ResponseBean<PageBean<ProjectQueryRes>> pageQueryControl(ProjectPageQueryReq projectPageQueryReq) {
        //1。分页查询所有供公司
        Integer page = projectPageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = projectPageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;
        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        if (projectPageQueryReq.getCreateTime() != null) {
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getName())) {
            queryWrapper.eq(Project::getCreateTime, projectPageQueryReq.getCreateTime());
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getOrderStatus())) {
            queryWrapper.eq(Project::getOrderStatus, projectPageQueryReq.getOrderStatus());
        }
        queryWrapper.eq(Project::getOrderStatus, ProjectStatus.RESERVES_ISSUED.getName());
        queryWrapper.eq(Project::getReserveStatus, ProjectStatus.APPROVE.getName());
        Page<Project> all = projectMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    ProjectQueryRes projectQueryRes = ProjectQueryRes
                            .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                            .build();
                    return projectQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<?> insertSubProject(ParentProjectInsertReq parentProjectInsertReq) {
        //查询父级项目
        Project parProject = projectMapper.selectById(parentProjectInsertReq.getId());
        if (parProject == null) {
            return ResponseBuilder.fail("父级项目不存在");
        }
        //更新父级项目
        BeanUtils.copyProperties(parentProjectInsertReq, parProject);
        projectMapper.updateById(parProject);
        //创建并关联父级项目
        List<SubProjectInsertReq> subProjectList = parentProjectInsertReq.getSubProjectList();
        for (SubProjectInsertReq subProjectInsertReq : subProjectList) {
            Project project = new Project();
            BeanUtils.copyProperties(subProjectInsertReq, project);
            project.setReserveStatus(ProjectStatus.APPROVE.getName());
            project.setOrderStatus(ProjectStatus.RESERVES_ISSUED_PASS.getName());
            project.setPurchaseStatus(ProjectStatus.TO_BE_REPORTED.getName());
            project.setDeleted(0);
            project.setParentId(parentProjectInsertReq.getId());
            project.setNo(ProjectNoUtil.getCode());
            Project save = projectRepository.save(project);
            //生成子项目采购审核阶段
            purchasingApprovalMapper.insertAll(save.getId());
            //关联文件
            if (subProjectInsertReq.getFileId() != null) {
                ProjectFile build = ProjectFile.builder()
                        .fileId(subProjectInsertReq.getFileId())
                        .projectId(save.getId())
                        .build();
                projectFileMapper.insert(build);
            }
        }
        return ResponseBuilder.ok();
    }

    @Override
    public ResponseBean<?> template() {
        String filePath = StringUtils.join(
                systemProperties.getDownloadPrefix() +
                        systemProperties.getFileTemplateDir() + "/项目储备导入模板.xlsx"
        );
        return ResponseBuilder.ok(filePath);
    }

    @Override
    public ResponseBean<?> giveExport(ProjectPageQueryReq projectPageQueryReq) {
        List<ProjectQueryRes> data = pageQueryControl(projectPageQueryReq).getData().getRow();
        String fileName = StringUtils.join(
                FileUtils.getFileNo(), SystemConstant.EXCEL_SUFFIX
        );
        String filePath = StringUtils.join(
                systemProperties.getFileStoreDirPath(), fileName
        );
        String fileUrl = StringUtils.join(
                systemProperties.getDownloadPrefix(), fileName
        );
        ExcelWriter excelWriter = EasyExcel.write(filePath, ProjectQueryRes.class).build();
        WriteSheet writeSheet = EasyExcel.writerSheet("储备下达数据").build();
        excelWriter.write(data, writeSheet);
        //关闭流
        excelWriter.finish();
        return ResponseBuilder.ok(fileUrl);
    }

    @Override
    public ResponseBean<PageBean<ProjectQueryRes>> pageQuerySubProject(ProjectPageQueryReq projectPageQueryReq) {
        Integer page = projectPageQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = projectPageQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;
        LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<>();
        if (projectPageQueryReq.getCreateTime() != null) {
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getName())) {
            queryWrapper.eq(Project::getCreateTime, projectPageQueryReq.getCreateTime());
        }
        if (StringUtils.isNotBlank(projectPageQueryReq.getOrderStatus())) {
            queryWrapper.eq(Project::getOrderStatus, projectPageQueryReq.getOrderStatus());
        }

        if (projectPageQueryReq.getSupplierId() != null) {
            queryWrapper.eq(Project::getSupplierId, projectPageQueryReq.getSupplierId());
        }

        if (projectPageQueryReq.getStatus() != null) {
            queryWrapper.eq(Project::getStatus, projectPageQueryReq.getStatus().getName());
        }
        queryWrapper.ne(Project::getParentId, 0);
        queryWrapper.eq(Project::getPurchaseStatus, ProjectStatus.CONTRACT_ENTRY.getName());
        Page<Project> all = projectMapper.selectPage(new Page<>(page, size), queryWrapper);
        return ResponseBuilder.ok(
                all,
                element -> {
                    ProjectQueryRes projectQueryRes = ProjectQueryRes
                            .builder()
                            .id(element.getId())
                            .capital(String.valueOf(element.getCapital()))
                            .content(element.getContent())
                            .cost(String.valueOf(element.getCost()))
                            .createTime(element.getCreateTime())
                            .level(element.getLevel())
                            .name(element.getName())
                            .progress(element.getProgress())
                            .projectClassify(element.getProjectClassify())
                            .specialtyClassify(element.getSpecialtyClassify())
                            .total(String.valueOf(element.getTotal()))
                            .totalInvestment(String.valueOf(element.getTotalInvestment()))
                            .twoClassify(element.getTwoClassify())
                            .unit(element.getUnit())
                            .orderStatus(String.valueOf(element.getOrderStatus()))
                            .reserveStatus(String.valueOf(element.getReserveStatus()))
                            .build();
                    return projectQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<OverviewRes> selectById(Integer id) {
        Project project = projectMapper.selectById(id);
        //项目基本信息
        OverviewRes overviewRes = EntityUtils.copyProperties(project, OverviewRes.class);
        //需求总数
        LambdaQueryWrapper<ProjectDemand> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ProjectDemand::getProjectId, project.getId());
        Integer count = projectDemandMapper.selectCount(queryWrapper);
        if (count != null) {
            overviewRes.setDemandTotal(count);
        }
        //项目阶段
        Integer typeId = project.getTypeId();
        LambdaQueryWrapper<ProjectTypeStage> typeWrapper = new LambdaQueryWrapper<>();
        typeWrapper.eq(ProjectTypeStage::getProjectTypeId, typeId);
        List<ProjectTypeStage> all = projectTypeStageMapper.selectList(typeWrapper);
        List<ProjectStageQueryRes> projectStagesList = new ArrayList<>();
        for (ProjectTypeStage typeStage : all) {
            ProjectStage projectStage = projectStageMapper.selectById(typeStage.getStageId());
            if (projectStage != null) {
                ProjectStageQueryRes stage = EntityUtils.copyProperties(projectStage, ProjectStageQueryRes.class);
                projectStagesList.add(stage);
            }
        }
        overviewRes.setStageList(projectStagesList);
        //项目里程碑
        List<MilestoneQueryRes> milestoneList = new ArrayList<>();
        LambdaQueryWrapper<Milestone> milestoneWrapper=new LambdaQueryWrapper<>();
        milestoneWrapper.eq(Milestone::getProjectId,project.getId());
        List<Milestone> milestones = milestoneMapper.selectList(milestoneWrapper);
        for (Milestone milestone : milestones) {
            MilestoneQueryRes milestoneQueryRes = EntityUtils.copyProperties(milestone, MilestoneQueryRes.class);
            milestoneList.add(milestoneQueryRes);
        }
        overviewRes.setMilestoneList(milestoneList);
        //采购审批
        List<PurchasingApprovalQueryRes> purchasingApprovalList = new ArrayList<>();
        LambdaQueryWrapper<PurchasingApproval> approvalWrapper=new LambdaQueryWrapper<>();
        approvalWrapper.eq(PurchasingApproval::getProjectId,project.getId());
        List<PurchasingApproval> approvals = purchasingApprovalMapper.selectList(approvalWrapper);
        for (PurchasingApproval approval : approvals) {
            PurchasingApprovalQueryRes approvalQueryRes = EntityUtils.copyProperties(approval, PurchasingApprovalQueryRes.class);
            purchasingApprovalList.add(approvalQueryRes);
        }
        overviewRes.setPurchasingApprovalList(purchasingApprovalList);
        //项目需求
        List<DemandLibraryPageQueryRes> demandList = new ArrayList<>();
        LambdaQueryWrapper<DemandLibrary> demandWrapper=new LambdaQueryWrapper<>();
        demandWrapper.eq(DemandLibrary::getAssociatedProjectId,project.getId());
        List<DemandLibrary> demandLibraries = demandLibraryMapper.selectList(demandWrapper);
        for (DemandLibrary demandLibrary : demandLibraries) {
            DemandLibraryPageQueryRes demandLibraryPageQueryRes = EntityUtils.copyProperties(demandLibrary, DemandLibraryPageQueryRes.class);
            demandList.add(demandLibraryPageQueryRes);
        }
        overviewRes.setDemandList(demandList);
        //项目财务
        List<FinancesQueryRes> financesList = new ArrayList<>();
        LambdaQueryWrapper<Finances> financesWrapper=new LambdaQueryWrapper<>();
        financesWrapper.eq(Finances::getProjectId,project.getId());
        List<Finances> finances = financesMapper.selectList(financesWrapper);
        for (Finances finance : finances) {
            FinancesQueryRes financesQueryRes = EntityUtils.copyProperties(finance, FinancesQueryRes.class);
            financesList.add(financesQueryRes);
        }
        overviewRes.setFinancesList(financesList);
        //项目成员
        List<ProjectMemberQueryRes> projectMemberList = new ArrayList<>();
        LambdaQueryWrapper<ProjectMember> memberWrapper=new LambdaQueryWrapper<>();
        memberWrapper.eq(ProjectMember::getProjectId,project.getId());
        List<ProjectMember> projectMembers = projectMemberMapper.selectList(memberWrapper);
        for (ProjectMember projectMember : projectMembers) {
            ProjectMemberQueryRes projectMemberQueryRes = EntityUtils.copyProperties(projectMember, ProjectMemberQueryRes.class);
            projectMemberList.add(projectMemberQueryRes);
        }
        overviewRes.setProjectMemberList(projectMemberList);
        //档案管理
        List<ContractQueryRes> contractList = new ArrayList<>();
        LambdaQueryWrapper<Contract> contractWrapper=new LambdaQueryWrapper<>();
        contractWrapper.eq(Contract::getProjectId,project.getId());
        List<Contract> contracts = contractMapper.selectList(contractWrapper);
        for (Contract contract : contracts) {
            ContractQueryRes contractQueryRes = EntityUtils.copyProperties(contract, ContractQueryRes.class);
            contractList.add(contractQueryRes);
        }
        overviewRes.setContractList(contractList);
        //评价
        List<CompanyEvaluationQueryRes> companyEvaluationList=new ArrayList<>();
        LambdaQueryWrapper<CompanyEvaluation> evaluationWrapper=new LambdaQueryWrapper<>();
        evaluationWrapper.eq(CompanyEvaluation::getProjectId,project.getId());
        List<CompanyEvaluation> companyEvaluations = companyEvaluationMapper.selectList(evaluationWrapper);
        for (CompanyEvaluation companyEvaluation : companyEvaluations) {
            CompanyEvaluationQueryRes companyEvaluationQueryRes = EntityUtils.copyProperties(companyEvaluation, CompanyEvaluationQueryRes.class);
            companyEvaluationList.add(companyEvaluationQueryRes);
        }
        overviewRes.setCompanyEvaluationList(companyEvaluationList);
        return ResponseBuilder.ok(overviewRes);
    }
}
