package com.migao.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.migao.config.authority.LoginUtils;
import com.migao.config.constant.SystemConstant;
import com.migao.config.properties.SystemProperties;
import com.migao.config.response.*;
import com.migao.entity.po.*;
import com.migao.entity.vo.req.CompanyEvaluationQuerysReq;
import com.migao.entity.vo.req.SupplierInsertReq;
import com.migao.entity.vo.req.SupplierQueryReq;
import com.migao.entity.vo.req.SupplierUpdateReq;
import com.migao.entity.vo.res.*;
import com.migao.mapper.*;
import com.migao.service.SupplierService;
import com.migao.util.EntityUtils;
import com.migao.util.FileUtils;
import com.migao.util.ProjectNoUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.List;

/**
 * @author dingsong
 */
@Slf4j
@Service
@Transactional(rollbackFor = {Exception.class})
public class SupplierServiceImpl extends ServiceImpl<SupplierMapper, Supplier> implements SupplierService {
    @Resource
    SupplierMapper supplierMapper;
    @Resource
    DictionaryMapper dictionaryMapper;
    @Resource
    PurchasingApprovalMapper approvalMapper;
    @Resource
    ProjectMapper projectMapper;
    @Resource
    CompanyEvaluationMapper companyEvaluationMapper;
    @Resource
    private SystemProperties systemProperties;

    @Override
    public ResponseBean<?> insert(SupplierInsertReq supplierInsertReq) {
        //用户信息
        User user = LoginUtils.getUser();

        Supplier supplier = EntityUtils.copyProperties(supplierInsertReq,Supplier.class);
        supplier.setSupplierCode(ProjectNoUtil.getCode());
        supplier.setDeleted(0);
        supplier.setCreateTime(LocalDateTime.now());
        supplier.setCreateUserId(user.getId());
        //Date转LocalDateTime
        Instant instant = supplierInsertReq.getCompanyCreateTime().toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        supplier.setCompanyCreateTime(instant.atZone(zoneId).toLocalDateTime());

        supplierMapper.insert(supplier);

        return ResponseBuilder.ok(200,"新增成功");
    }

    @Override
    public ResponseBean<?> deleteById(String id) {
        Integer idInt = Integer.parseInt(id);

        Supplier supplier = supplierMapper.selectById(idInt);
        supplier.setDeleted(1);
        supplierMapper.updateById(supplier);

        return ResponseBuilder.ok(200,"删除成功");
    }

    @Override
    public ResponseBean<?> updateById(SupplierUpdateReq supplierUpdateReq) {
        Supplier supplier = EntityUtils.copyProperties(supplierUpdateReq,Supplier.class);
        //Date转LocalDateTime
        Instant instant = supplierUpdateReq.getCompanyCreateTime().toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        supplier.setCompanyCreateTime(instant.atZone(zoneId).toLocalDateTime());
        supplier.setCreateTime(LocalDateTime.now());
        supplierMapper.updateById(supplier);
        return ResponseBuilder.ok(200,"更新成功");
    }

    @Override
    public ResponseBean<SupplierQueryRes> findById(String id) {
        Integer idInt = Integer.parseInt(id);
        Supplier supplier = supplierMapper.selectById(idInt);

        SupplierQueryRes supplierQueryRes = SupplierQueryRes
                .builder()
                .id(supplier.getId())
                .name(supplier.getName())
                .companyAlbum(supplier.getCompanyAlbum())
                .companySize(supplier.getCompanySize())
                .companyType(dictionaryMapper.selectById(supplier.getCompanyTypeId()).getValue())
                .supplierCode(supplier.getSupplierCode())
                .licenseNumber(supplier.getLicenseNumber())
                .legalPersonName(supplier.getLegalPersonName())
                .phoneOne(supplier.getPhoneOne())
                .phoneTwo(supplier.getPhoneTwo())
                .phone(supplier.getPhone())
                .address(supplier.getAddress())
                .money(supplier.getMoney())
                .businessScope(supplier.getBusinessScope())
                .remark(supplier.getRemark())
                .companyCreateTime(supplier.getCompanyCreateTime().toString())
                .intranetMail(supplier.getIntranetMail())
                .outsideMail(supplier.getOutsideMail())
                .principalName(supplier.getPrincipalName())
                .build();
        return ResponseBuilder.ok(supplierQueryRes);
    }

    @Override
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQuery(BasePageRequest basePageRequest) {
        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Supplier> supplier = new LambdaQueryWrapper<> ();
        supplier.eq(Supplier::getDeleted,0);
        supplier.orderByDesc(Supplier::getCreateTime);

        Page<Supplier> all = supplierMapper.selectPage(new Page<>(page, size),supplier);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    SupplierPageQueryRes supplierPageQueryRes = new SupplierPageQueryRes();
                    supplierPageQueryRes.setId(element.getId());
                    supplierPageQueryRes.setSupplierCode(element.getSupplierCode());
                    supplierPageQueryRes.setName(element.getName());
                    supplierPageQueryRes.setAddress(element.getAddress());
                    supplierPageQueryRes.setLicenseNumber(element.getLicenseNumber());
                    supplierPageQueryRes.setMoney(element.getMoney());
                    supplierPageQueryRes.setCompanyAlbum(element.getCompanyAlbum());
                    supplierPageQueryRes.setPrincipalName(element.getPrincipalName());
                    supplierPageQueryRes.setPhone(element.getPhone());

                    //所有项目
                    LambdaQueryWrapper<PurchasingApproval> postWrapper = new LambdaQueryWrapper<> ();
                    postWrapper.eq(PurchasingApproval::getSupplierId,element.getId());
                    postWrapper.orderByDesc(PurchasingApproval::getCreateTime);
                    List<PurchasingApproval> approvalList = approvalMapper.selectList(postWrapper);
                    List<Integer> projectId = new ArrayList<>();
                    approvalList.forEach(
                            x -> {
                                projectId.add(x.getProjectId());
                            }
                    );
                    SupplierProjectRes suOne = new SupplierProjectRes();
                    suOne.setProjectId(projectId);
                    suOne.setProjectSize(approvalList.size()+"");
                    supplierPageQueryRes.setSu1(suOne);

                    //已完成项目
                    List<Integer> projectIdTwo =  new ArrayList<>();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("已完成".equals(project.getStatus())){
                                    projectIdTwo.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suTwo  = new SupplierProjectRes();
                    suTwo.setProjectId(projectIdTwo);
                    suTwo.setProjectSize(projectIdTwo.size()+"");
                    supplierPageQueryRes.setSu2(suTwo);

                    //进行中项目
                    List<Integer> projectIdThree = new ArrayList<> ();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("进行中".equals(project.getStatus())){
                                    projectIdThree.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suThree = new SupplierProjectRes();
                    suThree.setProjectId(projectIdThree);
                    suThree.setProjectSize(projectIdThree.size()+"");
                    supplierPageQueryRes.setSu3(suThree);

                    //公司评价
                    List<Integer> projectsId  = new ArrayList<>();

                    LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<> ();
                    queryWrapper.eq(Project::getSupplierId,element.getId());
                    queryWrapper.eq(Project::getDeleted,0);
                    queryWrapper.orderByDesc(Project::getCreateTime);
                    List<Project> projectList = projectMapper.selectList(queryWrapper);
                    projectList.forEach(
                            x -> {
                                projectsId.add(x.getId());
                            }
                    );

                    //公司评价(然后通过项目id集合找到项目评价集合)
                    List<CompanyEvaluation> companyEvaluationList = new ArrayList<> ();
                    projectsId.forEach(
                            p -> {
                                CompanyEvaluation companyEvaluation =
                                        companyEvaluationMapper.selectById(p);
                                companyEvaluationList.add(companyEvaluation);
                            }
                    );
                    supplierPageQueryRes.setCompanyEvaluation(companyEvaluationList);

                    return supplierPageQueryRes;
                  }
                );
    }

    @Override
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQueryById(BasePageRequest basePageRequest) {
        //用户信息
        User user = LoginUtils.getUser();

        Integer page = basePageRequest.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = basePageRequest.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Supplier> supplier = new LambdaQueryWrapper<> ();
        supplier.eq(Supplier::getDeleted,0);
        supplier.eq(Supplier::getCreateUserId,user.getId());
        supplier.orderByDesc(Supplier::getCreateTime);

        Page<Supplier> all = supplierMapper.selectPage(new Page<>(page, size),supplier);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    SupplierPageQueryRes supplierPageQueryRes = new SupplierPageQueryRes();
                    supplierPageQueryRes.setId(element.getId());
                    supplierPageQueryRes.setSupplierCode(element.getSupplierCode());
                    supplierPageQueryRes.setName(element.getName());
                    supplierPageQueryRes.setAddress(element.getAddress());
                    supplierPageQueryRes.setLicenseNumber(element.getLicenseNumber());
                    supplierPageQueryRes.setMoney(element.getMoney());
                    supplierPageQueryRes.setCompanyAlbum(element.getCompanyAlbum());
                    supplierPageQueryRes.setPrincipalName(element.getPrincipalName());
                    supplierPageQueryRes.setPhone(element.getPhone());

                    //所有项目
                    LambdaQueryWrapper<PurchasingApproval> postWrapper = new LambdaQueryWrapper<> ();
                    postWrapper.eq(PurchasingApproval::getSupplierId,element.getId());
                    postWrapper.orderByDesc(PurchasingApproval::getCreateTime);
                    List<PurchasingApproval> approvalList = approvalMapper.selectList(postWrapper);
                    List<Integer> projectId = new ArrayList<>();
                    approvalList.forEach(
                            x -> {
                                projectId.add(x.getProjectId());
                            }
                    );
                    SupplierProjectRes suOne = new SupplierProjectRes();
                    suOne.setProjectId(projectId);
                    suOne.setProjectSize(approvalList.size()+"");
                    supplierPageQueryRes.setSu1(suOne);

                    //已完成项目
                    List<Integer> projectIdTwo =  new ArrayList<>();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("已完成".equals(project.getStatus())){
                                    projectIdTwo.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suTwo  = new SupplierProjectRes();
                    suTwo.setProjectId(projectIdTwo);
                    suTwo.setProjectSize(projectIdTwo.size()+"");
                    supplierPageQueryRes.setSu2(suTwo);

                    //进行中项目
                    List<Integer> projectIdThree = new ArrayList<> ();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("进行中".equals(project.getStatus())){
                                    projectIdThree.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suThree = new SupplierProjectRes();
                    suThree.setProjectId(projectIdThree);
                    suThree.setProjectSize(projectIdThree.size()+"");
                    supplierPageQueryRes.setSu3(suThree);

                    //公司评价(先通过供应商id找到所有的项目id)
                    List<Integer> projectsId  = new ArrayList<>();

                    LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<> ();
                    queryWrapper.eq(Project::getSupplierId,element.getId());
                    queryWrapper.eq(Project::getDeleted,0);
                    queryWrapper.orderByDesc(Project::getCreateTime);
                    List<Project> projectList = projectMapper.selectList(queryWrapper);
                    projectList.forEach(
                            x -> {
                                projectsId.add(x.getId());
                            }
                    );

                    //公司评价(然后通过项目id集合找到项目评价集合)
                    List<CompanyEvaluation> companyEvaluationList = new ArrayList<> ();
                    projectsId.forEach(
                            p -> {
                                CompanyEvaluation companyEvaluation =
                                        companyEvaluationMapper.selectById(p);
                                companyEvaluationList.add(companyEvaluation);
                            }
                    );
                    supplierPageQueryRes.setCompanyEvaluation(companyEvaluationList);

                    return supplierPageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQueryOne(SupplierQueryReq supplierQueryReq) {

        Integer page = supplierQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = supplierQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Supplier> supplier = new LambdaQueryWrapper<> ();
        supplier.eq(Supplier::getDeleted,0);
        if(StringUtils.isNotBlank(supplierQueryReq.getKey())){
            supplier.like(Supplier::getName,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or1 = supplier.or();
            or1.like(Supplier::getCompanyAlbum,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or2 = or1.or();
            or2.like(Supplier::getCompanySize,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or3 = or2.or();
            or3.like(Supplier::getSupplierCode,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or4 = or3.or();
            or4.like(Supplier::getLicenseNumber,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or5 = or4.or();
            or5.like(Supplier::getLegalPersonName,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or6 = or5.or();
            or6.like(Supplier::getPrincipalName,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or7 = or6.or();
            or7.like(Supplier::getAddress,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or8 = or7.or();
            or8.like(Supplier::getMoney,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or9 = or8.or();
            or9.like(Supplier::getBusinessScope,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or10 = or9.or();
            or10.like(Supplier::getRemark,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or11 = or10.or();
            or11.like(Supplier::getIntranetMail,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or12 = or11.or();
            or12.like(Supplier::getOutsideMail,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or13 = or12.or();
            or13.like(Supplier::getOpeningBankNumber,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or14 = or13.or();
            or14.like(Supplier::getOpeningBank,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or15 = or14.or();
            or15.like(Supplier::getBankAccount,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or16 = or15.or();
            or16.like(Supplier::getOpeningBankNumberD,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or17 = or16.or();
            or17.like(Supplier::getOpeningBankD,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or18 = or17.or();
            or18.like(Supplier::getBankAccountD,supplierQueryReq.getKey());
        }
        supplier.orderByDesc(Supplier::getCreateTime);

        Page<Supplier> all = supplierMapper.selectPage(new Page<>(page, size),supplier);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    SupplierPageQueryRes supplierPageQueryRes = new SupplierPageQueryRes();
                    supplierPageQueryRes.setId(element.getId());
                    supplierPageQueryRes.setSupplierCode(element.getSupplierCode());
                    supplierPageQueryRes.setName(element.getName());
                    supplierPageQueryRes.setAddress(element.getAddress());
                    supplierPageQueryRes.setLicenseNumber(element.getLicenseNumber());
                    supplierPageQueryRes.setMoney(element.getMoney());
                    supplierPageQueryRes.setCompanyAlbum(element.getCompanyAlbum());
                    supplierPageQueryRes.setPrincipalName(element.getPrincipalName());
                    supplierPageQueryRes.setPhone(element.getPhone());

                    //所有项目
                    LambdaQueryWrapper<PurchasingApproval> postWrapper = new LambdaQueryWrapper<> ();
                    postWrapper.eq(PurchasingApproval::getSupplierId,element.getId());
                    postWrapper.orderByDesc(PurchasingApproval::getCreateTime);
                    List<PurchasingApproval> approvalList = approvalMapper.selectList(postWrapper);
                    List<Integer> projectId = new ArrayList<>();
                    approvalList.forEach(
                            x -> {
                                projectId.add(x.getProjectId());
                            }
                    );
                    SupplierProjectRes suOne = new SupplierProjectRes();
                    suOne.setProjectId(projectId);
                    suOne.setProjectSize(approvalList.size()+"");
                    supplierPageQueryRes.setSu1(suOne);

                    //已完成项目
                    List<Integer> projectIdTwo =  new ArrayList<>();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("已完成".equals(project.getStatus())){
                                    projectIdTwo.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suTwo  = new SupplierProjectRes();
                    suTwo.setProjectId(projectIdTwo);
                    suTwo.setProjectSize(projectIdTwo.size()+"");
                    supplierPageQueryRes.setSu2(suTwo);

                    //进行中项目
                    List<Integer> projectIdThree = new ArrayList<> ();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("进行中".equals(project.getStatus())){
                                    projectIdThree.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suThree = new SupplierProjectRes();
                    suThree.setProjectId(projectIdThree);
                    suThree.setProjectSize(projectIdThree.size()+"");
                    supplierPageQueryRes.setSu3(suThree);

                    //公司评价
                    List<Integer> projectsId  = new ArrayList<>();

                    LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<> ();
                    queryWrapper.eq(Project::getSupplierId,element.getId());
                    queryWrapper.eq(Project::getDeleted,0);
                    queryWrapper.orderByDesc(Project::getCreateTime);
                    List<Project> projectList = projectMapper.selectList(queryWrapper);
                    projectList.forEach(
                            x -> {
                                projectsId.add(x.getId());
                            }
                    );

                    //公司评价(然后通过项目id集合找到项目评价集合)
                    List<CompanyEvaluation> companyEvaluationList = new ArrayList<> ();
                    projectsId.forEach(
                            p -> {
                                CompanyEvaluation companyEvaluation =
                                        companyEvaluationMapper.selectById(p);
                                companyEvaluationList.add(companyEvaluation);
                            }
                    );
                    supplierPageQueryRes.setCompanyEvaluation(companyEvaluationList);

                    return supplierPageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<SupplierPageQueryRes>> pageQueryOneById(SupplierQueryReq supplierQueryReq) {
        //用户信息
        User user = LoginUtils.getUser();

        Integer page = supplierQueryReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = supplierQueryReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<Supplier> supplier = new LambdaQueryWrapper<> ();
        supplier.eq(Supplier::getDeleted,0);
        supplier.eq(Supplier::getCreateUserId,user.getId());
        if(StringUtils.isNotBlank(supplierQueryReq.getKey())){
            supplier.like(Supplier::getName,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or1 = supplier.or();
            or1.like(Supplier::getCompanyAlbum,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or2 = or1.or();
            or2.like(Supplier::getCompanySize,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or3 = or2.or();
            or3.like(Supplier::getSupplierCode,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or4 = or3.or();
            or4.like(Supplier::getLicenseNumber,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or5 = or4.or();
            or5.like(Supplier::getLegalPersonName,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or6 = or5.or();
            or6.like(Supplier::getPrincipalName,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or7 = or6.or();
            or7.like(Supplier::getAddress,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or8 = or7.or();
            or8.like(Supplier::getMoney,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or9 = or8.or();
            or9.like(Supplier::getBusinessScope,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or10 = or9.or();
            or10.like(Supplier::getRemark,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or11 = or10.or();
            or11.like(Supplier::getIntranetMail,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or12 = or11.or();
            or12.like(Supplier::getOutsideMail,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or13 = or12.or();
            or13.like(Supplier::getOpeningBankNumber,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or14 = or13.or();
            or14.like(Supplier::getOpeningBank,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or15 = or14.or();
            or15.like(Supplier::getBankAccount,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or16 = or15.or();
            or16.like(Supplier::getOpeningBankNumberD,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or17 = or16.or();
            or17.like(Supplier::getOpeningBankD,supplierQueryReq.getKey());

            LambdaQueryWrapper<Supplier> or18 = or17.or();
            or18.like(Supplier::getBankAccountD,supplierQueryReq.getKey());
        }
        supplier.orderByDesc(Supplier::getCreateTime);

        Page<Supplier> all = supplierMapper.selectPage(new Page<>(page, size),supplier);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    SupplierPageQueryRes supplierPageQueryRes = new SupplierPageQueryRes();
                    supplierPageQueryRes.setId(element.getId());
                    supplierPageQueryRes.setSupplierCode(element.getSupplierCode());
                    supplierPageQueryRes.setName(element.getName());
                    supplierPageQueryRes.setAddress(element.getAddress());
                    supplierPageQueryRes.setLicenseNumber(element.getLicenseNumber());
                    supplierPageQueryRes.setMoney(element.getMoney());
                    supplierPageQueryRes.setCompanyAlbum(element.getCompanyAlbum());
                    supplierPageQueryRes.setPrincipalName(element.getPrincipalName());
                    supplierPageQueryRes.setPhone(element.getPhone());

                    //所有项目
                    LambdaQueryWrapper<PurchasingApproval> postWrapper = new LambdaQueryWrapper<> ();
                    postWrapper.eq(PurchasingApproval::getSupplierId,element.getId());
                    postWrapper.orderByDesc(PurchasingApproval::getCreateTime);
                    List<PurchasingApproval> approvalList = approvalMapper.selectList(postWrapper);
                    List<Integer> projectId = new ArrayList<>();
                    approvalList.forEach(
                            x -> {
                                projectId.add(x.getProjectId());
                            }
                    );
                    SupplierProjectRes suOne = new SupplierProjectRes();
                    suOne.setProjectId(projectId);
                    suOne.setProjectSize(approvalList.size()+"");
                    supplierPageQueryRes.setSu1(suOne);

                    //已完成项目
                    List<Integer> projectIdTwo =  new ArrayList<>();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("已完成".equals(project.getStatus())){
                                    projectIdTwo.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suTwo  = new SupplierProjectRes();
                    suTwo.setProjectId(projectIdTwo);
                    suTwo.setProjectSize(projectIdTwo.size()+"");
                    supplierPageQueryRes.setSu2(suTwo);

                    //进行中项目
                    List<Integer> projectIdThree = new ArrayList<> ();
                    projectId.forEach(
                            x -> {
                                Project project = projectMapper.selectById(x);
                                if("进行中".equals(project.getStatus())){
                                    projectIdThree.add(x);
                                }
                            }
                    );
                    SupplierProjectRes suThree = new SupplierProjectRes();
                    suThree.setProjectId(projectIdThree);
                    suThree.setProjectSize(projectIdThree.size()+"");
                    supplierPageQueryRes.setSu3(suThree);

                    //公司评价
                    List<Integer> projectsId  = new ArrayList<>();

                    LambdaQueryWrapper<Project> queryWrapper = new LambdaQueryWrapper<> ();
                    queryWrapper.eq(Project::getSupplierId,element.getId());
                    queryWrapper.eq(Project::getDeleted,0);
                    queryWrapper.orderByDesc(Project::getCreateTime);
                    List<Project> projectList = projectMapper.selectList(queryWrapper);
                    projectList.forEach(
                            x -> {
                                projectsId.add(x.getId());
                            }
                    );

                    //公司评价(然后通过项目id集合找到项目评价集合)
                    List<CompanyEvaluation> companyEvaluationList = new ArrayList<> ();
                    projectsId.forEach(
                            p -> {
                                CompanyEvaluation companyEvaluation =
                                        companyEvaluationMapper.selectById(p);
                                companyEvaluationList.add(companyEvaluation);
                            }
                    );
                    supplierPageQueryRes.setCompanyEvaluation(companyEvaluationList);

                    return supplierPageQueryRes;
                }
        );
    }

    @Override
    public ResponseBean<PageBeanOne<CompanyEvaluationQueryRes>> pageEvaluation(CompanyEvaluationQuerysReq cReq) {
        Integer page = cReq.getPage();
        page = page == null ? 1 : page <= 0 ? 1 : page;
        Integer size = cReq.getSize();
        size = size == null ? 10 : size <= 0 ? 10 : size;

        LambdaQueryWrapper<CompanyEvaluation> cQueryWrapper = new LambdaQueryWrapper<>();

        for(Integer i : cReq.getId()){
            cQueryWrapper.eq(CompanyEvaluation::getId,i);
        }
        Page<CompanyEvaluation> all = companyEvaluationMapper.selectPage(new Page<>(page, size),cQueryWrapper);

        return ResponseBuilderOne.ok(
                all,
                element -> {
                    CompanyEvaluationQueryRes cRes = CompanyEvaluationQueryRes
                            .builder()
                            .content(element.getContent())
                            .createTime(element.getCreateTime())
                            .give(element.getGive())
                            .username(element.getUsername())
                            .id(element.getId())
                            .build();
                    return  cRes;
                  }
                );
    }

    @Override
    public ResponseBean<?> exportSupplier() {
        LambdaQueryWrapper<Supplier> sqlWrapper  = Wrappers.lambdaQuery();
        List<Supplier> supplierList = supplierMapper.selectList(sqlWrapper);
        List<SupplierExportRes> res = new ArrayList<>();

        supplierList.forEach(
                x -> {
                    SupplierExportRes supplierExportRes = SupplierExportRes
                            .builder()
                            .id(x.getId())
                            .name(x.getName())
                            .companyAlbum(x.getCompanyAlbum())
                            .companySize(x.getCompanySize())
                            .companyType(dictionaryMapper.selectById(x.getCompanyTypeId()).getValue())
                            .supplierCode(x.getSupplierCode())
                            .licenseNumber(x.getLicenseNumber())
                            .legalPersonName(x.getLegalPersonName())
                            .phoneOne(x.getPhoneOne().toString())
                            .phoneTwo(x.getPhoneTwo().toString())
                            .principalName(x.getPrincipalName())
                            .phone(x.getPhone().toString())
                            .address(x.getAddress())
                            .money(x.getMoney())
                            .businessScope(x.getBusinessScope())
                            .remark(x.getRemark())
                            .companyCreateTime(x.getCompanyCreateTime())
                            .intranetMail(x.getIntranetMail())
                            .outsideMail(x.getOutsideMail())
                            .openingBankNumber(x.getOpeningBankNumber())
                            .openingBank(x.getOpeningBank())
                            .bankAccount(x.getBankAccount())
                            .openingBankNumberD(x.getOpeningBankNumberD())
                            .openingBankD(x.getOpeningBankD())
                            .bankAccountD(x.getBankAccountD())
                            .build();
                    res.add(supplierExportRes);
                }
        );

        String fileName = StringUtils.join(
                FileUtils.getFileNo(), SystemConstant.EXCEL_SUFFIX
        );
        String filePath = StringUtils.join(
                systemProperties.getFileStoreDirPath(), fileName
        );
        String fileUrl = StringUtils.join(
                systemProperties.getDownloadPrefix(), fileName
        );

        ExcelWriter excelWriter = EasyExcel.write(filePath, SupplierExportRes.class).build();
        WriteSheet writeSheet = EasyExcel.writerSheet("供应商数据").build();
        excelWriter.write(res, writeSheet);
        //关闭流
        excelWriter.finish();
        return ResponseBuilder.ok(fileUrl);
    }

    @Override
    public ResponseBean<List<DictionaryListRes>> companyTypeClassify() {
        String value = "COMPANY_TYPE_CLASSIFY";
        List<DictionaryListRes> all = dictionaryMapper.findByValue(value);
        return ResponseBuilder.ok(all);
    }


}
